local playerClass = select(2,UnitClass("player"))
if playerClass ~= "ROGUE" then return end

if not DrDamage then DrDamage = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "AceHook-2.1", "FuBarPlugin-2.0") end
local L = AceLibrary("AceLocale-2.2"):new("DrDamage")
local BS = AceLibrary("Babble-Spell-2.2")
local SEA = AceLibrary("SpecialEvents-Aura-2.0")
local GT = AceLibrary("Gratuity-2.0")
local Deformat = AceLibrary("Deformat-2.0")


--General
local type = type
local pairs = pairs
local ipairs = ipairs
local tonumber = tonumber
local math_abs = math.abs
local math_floor = math.floor
local math_ceil = math.ceil
local string_match = string.match
local select = select
local UnitLevel = UnitLevel

--Melee
local GetCritChance = GetCritChance
local GetCombatRatingBonus = GetCombatRatingBonus
local GetItemInfo = GetItemInfo
local GetInventoryItemLink = GetInventoryItemLink
local GetInventorySlotInfo = GetInventorySlotInfo

local UnitRangedDamage = UnitRangedDamage
local UnitDamage = UnitDamage
local UnitAttackSpeed = UnitAttackSpeed
local UnitAttackPower = UnitAttackPower
local UnitRangedAttackPower = UnitRangedAttackPower

local function DrD_ClearTable( table )
	for k in pairs( table ) do
		table[k] = nil
	end
end

local function DrD_Round(x, y)
	return math_floor( x * 10 ^ y + 0.5 ) / 10 ^ y
end

local function DrD_MatchData( data, ... )
	if not data or not ... then
		return false
	end

	if type( data ) == "table" then
		for _, dataName in ipairs( data ) do
			for i = 1, select('#', ...) do
				if dataName == select(i, ...) then
					return true
				end
			end
		end
	else	
		for i = 1, select('#', ...) do
			if data == select(i, ...) then
				return true
			end
		end
	end
	
	return false
end

DrDamage:RegisterDB("DrDamageDB");
DrDamage:RegisterDefaults("profile", {
} )

local settings

function DrDamage:OnEnable()
	DrDamage:Melee_OnEnable()
end

function DrDamage:Melee_OnEnable()
	self:PlayerData()
	self:Melee_OnProfileEnable()
end

function DrDamage:Melee_OnProfileEnable()
	settings = self.db.profile
end

local options = {}

--DrDamage.OnMenuRequest = options
DrDamage.hasIcon = "Interface\\Icons\\Ability_DualWield"

local normalizationTable = {
	["Daggers"] = 1.7,
	["One-Handed Axes"] = 2.4,
	["One-Handed Maces"] = 2.4,
	["One-Handed Swords"] = 2.4,
	["Fist Weapons"] = 2.4,	
	["Bows"] = 2.8,
	["Crossbows"] = 2.8,
	["Guns"] = 2.8,
	["Two-Handed Axes"] = 3.3,
	["Two-Handed Maces"] = 3.3,
	["Two-Handed Swords"] = 3.3,	
	["Polearms"] = 3.3,
	["Staves"] = 3.3,
	["Fishing Pole"] = 3.3
}

function DrDamage:GetAP()
	local baseAP, posBuff, negBuff = UnitAttackPower("player")
	return baseAP + posBuff + negBuff
end

function DrDamage:GetRAP()
	local baseAP, posBuff, negBuff = UnitRangedAttackPower("player")
	return baseAP + posBuff + negBuff	
end

function DrDamage:GetNormM()
	local mainLink = GetInventoryItemLink("player",GetInventorySlotInfo("MainHandSlot"))
	local offLink = GetInventoryItemLink("player",GetInventorySlotInfo("SecondaryHandSlot"))
	local mainH, offH
	if mainLink then
		mainH = select(7,GetItemInfo(mainLink))
	else
		mainH = "None"
	end
	if offLink then
		offH = select(7,GetItemInfo(offLink))
	else
		offH = "None"
	end
	return normalizationTable[mainH], normalizationTable[offH]
end

function DrDamage:BaseDamage(AP,ranged)
	local min, max, omin, omax, mod, spd, ospd
	if ranged then
		spd, min, max, _, _, mod = UnitRangedDamage("player")
	else
		min, max, omin, omax, _, _, mod = UnitDamage("player")
		spd, ospd = UnitAttackSpeed("player")	
	end
	
	min = min / mod - (AP / 14) * spd
	max = max / mod - (AP / 14) * spd
		
	if not ranged and OffhandHasWeapon() then
		omin = math.floor((omin/0.5)/mod - (AP / 14) * ospd + 0.5)
		omax = math.floor((omax/0.5)/mod - (AP / 14) * ospd + 0.5)
	end
	
	return math.floor(min+0.5), math.floor(max+0.5), omin, omax, pmod
end

function DrDamage:WeaponDamage(AP, ws, ranged)
	local min, max, omin, omax, mod = DrDamage:BaseDamage(AP, ranged)
	local normM
	
	if ranged then
		normM = UnitRangedDamage("player")
	else
		normM, onormM = UnitAttackSpeed("player")
	end
	
	if not ws then
		mh, oh = DrDamage:GetNormM()
		normM = mh or normM
		onormM = oh or onormM
	end
	
	min = (min + (AP / 14) * normM) * mod
	max = (max + (AP / 14) * normM) * mod
	
	if onormM and omin and omax then
		omin = (omin + (AP / 14) * onormM) * 0.5 * mod
		omax = (omax + (AP / 14) * onormM) * 0.5 * mod
	end
	
	return min, max, omin, omax
end

local calculation = {}
function DrDamage:MeleeCalc( --[[action,--]] lAction, rank, tooltip )

	DrD_ClearTable( calculation )
	
	local temp = DrDamage.spellInfo[lAction]
	local baseAction = temp[0]
	local action = temp[rank]
	local ranged, AP
	local calculation.hitPerc = GetCombatRatingBonus(combatRatingIdentifier)
	 
	if baseAction.School == "Ranged" then
		ranged = true
		AP = DrDamage:GetRAP()
		calculation.critPerc = GetRangedCritChance()
		calculation.hitPerc = GetCombatRatingBonus(7)
		calculation.haste = GetCombatRatingBonus(18)
	else
		AP = DrDamage:GetAP()
		calculation.critPerc = GetCritChance()
		calculation.hitPerc = GetCombatRatingBonus(6)
		calculation.haste = GetCombatRatingBonus(19)
	end
	
	if baseAction.WeaponDamage then
		local min, max, omin, omax = DrDamage:WeaponDamage( AP, baseAction.NoNormalization, ranged )
		min = min * baseAction.WeaponDamage
		max = max * baseAction.WeaponDamage
		if baseAction.DualAttack and OffhandHasWeapon() then
			min = min + omin * baseAction.WeaponDamage + action[1]
			max = max + omax * baseAction.WeaponDamage + action[2] or action[1]
		end
		calculation.minDam, calculation.maxDam = min, max
	else
		calculation.minDam = 0
		calculation.maxDam = 0
	end	
	
	calculation.minDam = calculation.minDam + action[1]
	calculation.maxDam = calculation.maxDam + action[2] or action[1]
	calculation.eDuration = baseAction.eDuration or 0
	
	if baseAction.ComboPoints then
		local cp = GetComboPoints()
		
		if cp > 0 then
			if action.PerCombo then
				calculation.minDam = calculation.minDam + action.PerCombo * cp
				calculation.maxDam = calculation.maxDam + action.PerCombo * cp
			end
			if baseAction.ExtraPerCombo then
				calculation.minDam = calculation.minDam + action.Extra * baseAction.ExtraPerCombo[cp]
				calculation.maxDam = calculation.maxDam + action.Extra * baseAction.ExtraPerCombo[cp]
			end
			if baseAction.APBonus then
				if type( baseAction.APBonus ) == "table" then
					calculation.minDam = calculation.minDam + baseAction.APBonus[cp] * AP
					calculation.maxDam = calculation.minDam + baseAction.APBonus[cp] * AP
				else
					calculation.minDam = calculation.minDam + baseAction.APBonus * cp * AP
					calculation.maxDam = calculation.minDam + baseAction.APBonus * cp * AP
				end
			end
			if baseAction.DurationPerCombo then
				calculation.eDuration = calculation.eDuration + baseAction.DurationPerCombo * cp
			end
		else
			calculation.minDam = 0
			calculation.maxDam = 0
		end
	else
		if baseAction.APBonus then
			calculation.minDam = calculation.minDam + baseAction.APBonus * AP
			calculation.maxDam = calculation.maxDam + baseAction.APBonus * AP		
		end
	end
	
	calculation.avg = ((calculation.minDam + calculation.maxDam) / 2)
	
	if not baseAction.NoCrits then
		if calculation.critPerc > 100 then
			calculation.critPerc = 100
		end

		calculation.avg = calculation.avg + (calculation.critPerc / 100) *  calculation.avg
	end
	
	calculation.avg = math.floor(calculation.avg + 0.5)
	calculation.minDam = math.floor(calculation.minDam + 0.5)
	calculation.maxDam = math.floor(calculation.maxDam + 0.5)
	calculation.AP = AP
	
	return calculation.avg, calculation.minDam, calculation.maxDam, calculation
end